/*
  fix_FFT_EEG_DC_128Hz.ino
  Code by by LINGIB
  
  8/12/2020 MTS Added HC05,removed display
  9/4 MTS 1Hz bins, 

  
  https://www.instructables.com/member/lingib/instructables/
  Last update 7 August 2020

  This code:
   - extracts the frequencies in the range DC-64Hz from a complex waveform and
   - displays their amplitudes against time.
   - each of the 32 bins is 2Hz wide
   - the DC offset is automatically removed from bin[0]

  These libraries must be installed for the code to compile:
  https://github.com/adafruit/Adafruit-GFX-Library
  https://github.com/adafruit/Adafruit_ILI9341
  https://github.com/adafruit/Adafruit_BusIO

  -------------
  TERMS OF USE:
  -------------
  The software is provided "AS IS", without any warranty of any kind, express or implied,
  including but not limited to the warranties of mechantability, fitness for a particular
  purpose and noninfringement. In no event shall the authors or copyright holders be liable
  for any claim, damages or other liability, whether in an action of contract, tort or
  otherwise, arising from, out of or in connection with the software or the use or other
  dealings in the software.
*/


////////////////// fix_FFT ///////////////////////

#include "fix_fft.h"

// ----- analog input
#define adc_input A0

// ----- FFT
const unsigned long sample_freq = 128;                      // Hz (Nyquist freq = 64Hz)
const short n = 128;                                         // number of data points (must be power of 2)
const unsigned long sample_period = 1000000 / sample_freq;  // uS
const short m = (short(log10(n) / log10(2)));               // equivalent to m=log2(n) from which 2^m=n
char data[n];                                               // real values
char im[n];                                                 // imaginary values

int C0, delta, theta, alpha, beta, gamma;

short dc_offset;
short dc_counter = 5;
bool data_valid = false;

// ----- timers
unsigned long Timer1;
unsigned long Timer2;

// ----------------------
//  setup()
// ----------------------
void setup()
{
  Serial.begin(19200);
  Serial1.begin(19200);
  pinMode(adc_input, INPUT);


  // ----- start timers
  Timer1 = micros() + sample_period;                        // set loop()timer
  Timer2 = micros() + 1000000L;                             // print results every second
}

// ----------------------
//  loop()
// ----------------------
void loop(void)
{
  // ----- locals
  static short i = 0;                                       // sample counter
  
  // ----- loop() control
  while (micros() < Timer1);                                // wheel-spin until next sample due
  Timer1 += sample_period;                                  // reset loop timer

  if (i < n) {

    ////////////////////////
    //  Acquire the data  //
    ////////////////////////
    
    // ----- take 'n' samples 
    short value = analogRead(adc_input);
    data[i] = map(value, 0, 1023, 0, 255);
    im[i] = 0;
    i++;
  }
  else {

    ////////////////////////
    //  Display the data  //  
    ////////////////////////

    // ----- Display data once per second
    if (micros() > Timer2) {
      Timer2 += 1000000L;

      // ----- process data
      fix_fft(data, im, m, 0);

      // ----- calculate amplitudes
      for (short i = 0; i < n / 2; i++) {                         // ignore image data beyond n/2
        data[i] = (sqrt(data[i] * data[i] + im[i] * im[i]));
      }

      // ----- capture dc offset
      if (!data_valid && (dc_counter > 0)) {
        dc_counter--;
        if (dc_counter < 1) data_valid = true;
        dc_offset = data[0];

        //Serial.println(dc_offset);                              // debug
      }

      // ----- display the data
      if (data_valid) {

        // ----- remove dc offset
        data[0] -= dc_offset;

        // ----- send all bins to Processing
        C0 = (int)data[0];

        delta = 0;
        theta = 0;
        alpha = 0;
        beta = 0;
        gamma = 0;
        for (int i = 1; i < 4; i++) {
          delta += (int)data[i];
        }
        for (int i = 4; i < 8; i++) {
          theta += (int)data[i];
        }
        for (int i = 8; i < 13; i++) {
          alpha += (int)data[i];
        }
        for (int i = 13; i < 32; i++) {
          beta += (int)data[i];
        }
        for (int i = 32; i < 64; i++) {
          gamma += (int)data[i];
        }

        String message = "";
        message+= C0;
        message+= ',';
        message+= delta;
        message+= ',';
        message+= theta;
        message+= ',';
        message+= alpha;
        message+= ',';
        message+= beta;
        message+= ',';
        message+= gamma;
 
        Serial.println(message);
		
        Serial1.print("$");     // Start char
        Serial1.println(message);
      }

    }

    //////////////////////////////////////////////////////////////////
    //  Prepare for next sample
    //////////////////////////////////////////////////////////////////
    
    // ----- prepare to acquire another data set
    i = 0;                                                      // reset data acquistition
    Timer1 = micros() + sample_period;                          // reset loop() timer
  }
}
